#=========================================================================
# R-file for simulation experiments (Tab. 1)      
# author: Wolfgang Hess
# ========================================================================

library(gamlss.mx)

set.seed(4537269)
iterations = 100

n <- 5000 #number of individuals
k <- 12  #maximum number of periods before the spell is artificially censored
nk <- n*k
id <- rep(1:n, each=k)

for (i in 1:iterations) {
  
  x1 <- rep(rnorm(n = n, mean = 0, sd = 1), each=k)   
  x2 <- rep(rgamma(n = n, shape=1)-1, each=k)   #gamma distributed x with mean=0 and sd=1
  t <- rep(1:k, times=n)
  lnt <- log(t)  #baseline hazard
  h <- 1-(1+5/exp(-(x1+x2-lnt)))^(-1/5)   #grouped hazard
  
  y <- rep(0, times=nk)
  for (j in 1:nk) {
    if (h[j]>runif(1)) {
      y[j]<-1
    }
  }
  summary(h)
  summary(y) #means should be roughly equal!
  
  
  #generating duration dummies
  #---------------------------
  t2 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==2) {
      t2[j]<-1
    }
  }
  t3 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==3) {
      t3[j]<-1
    }
  }
  t4 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==4) {
      t4[j]<-1
    }
  }
  t5 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==5) {
      t5[j]<-1
    }
  }
  t6<- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==6) {
      t6[j]<-1
    }
  }
  t7 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==7) {
      t7[j]<-1
    }
  }
  t8 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==8) {
      t8[j]<-1
    }
  }
  t9 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==9) {
      t9[j]<-1
    }
  }
  t10 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==10) {
      t10[j]<-1
    }
  }
  t11 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==11) {
      t11[j]<-1
    }
  }
  t12 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==12) {
      t12[j]<-1
    }
  }
  
  
  #dropping obs. after the first y_it=1
  #------------------------------------
  temp <- rep(0, times=nk)
  for (l in seq(1,nk,by=k)) {
    m<-l+k-1
    temp[l:m]<-cumsum(y[l:m])
  }
  drop <- rep(0, times=nk)
  for (j in 1:nk) {
    if (temp[j]>1||(temp[j]==1&&y[j]==0)) {
      drop[j]<-1
    }
  }
  dat <- data.frame(id,y,x1,x2,t,lnt,t2,t3,t4,t5,t6,t7,t8,t9,t10,t11,t12,drop,h)
  dat <- dat[dat$drop==0,]
  assign(paste("data.set", i, sep = ""), dat)
  
  print(i)
}

tot<-nrow(data.set1)
for (i in 2:100){
  tot<-tot+nrow(get(paste("data.set", i, sep = "")))
}
tot/100

#save(list=ls(pattern="^data.set"), file="sim4.RData")
rm(list=setdiff(ls(), c("iterations", ls(pattern="^data.set"))))

burr <- function(xi=1) 
{ 
  linkfun <- function(mu) log(((1-mu)^(-xi)-1)/xi) 
  linkinv <- function(eta) 1-(1+xi*exp(eta))^(-1/xi) 
  mu.eta <- function(eta) exp(eta)*(1+xi*exp(eta))^(-1/xi-1) 
  valideta <- function(eta) TRUE 
  link <- paste("burr(", xi, ")", sep="") 
  structure(list(linkfun = linkfun, linkinv = linkinv, 
                 mu.eta = mu.eta, valideta = valideta, name = link), 
            class = "link-gamlss") 
}

for (j in 0:3){
  assign(paste("coef.matrix", j, sep = ""), 
         matrix(data = NA, nrow = iterations, ncol = 14,
                dimnames=list(1:iterations,c("cons", "t2", "t3", "t4", "t5",
                           "t6", "t7", "t8", "t9", "t10", 
                           "t11", "t12", "x1", "x2"))))
}

for (i in 1:iterations) {
  dat<-get(paste("data.set", i, sep = ""))
  
  try({model0 <- gamlssNP(y ~ t2+t3+t4+t5+t6+t7+t8+t9+t10+t11+t12+x1+x2, random=~1|id, family = BI(cloglog), 
                      data=dat, K=1, mixture="gq", tol=0.5, control = NP.control(EMcc=0.001, trace=FALSE), 
                      g.control = gamlss.control(trace=FALSE))
  coef.matrix0[i,] <- as.vector(coef(model0))},TRUE)
  
  try({model1 <- gamlssNP(y ~ t2+t3+t4+t5+t6+t7+t8+t9+t10+t11+t12+x1+x2, random=~1|id, family = BI(logit), 
                          data=dat, K=1, mixture="gq", tol=0.5, control = NP.control(EMcc=0.001, trace=FALSE), 
                          g.control = gamlss.control(trace=FALSE))
       coef.matrix1[i,] <- as.vector(coef(model1))},TRUE)
  
  try({model2 <- gamlssNP(y ~ t2+t3+t4+t5+t6+t7+t8+t9+t10+t11+t12+x1+x2, random=~1|id, family = BI(burr(xi=5)), 
                          data=dat, K=1, mixture="gq", tol=0.5, control = NP.control(EMcc=0.001, trace=FALSE), 
                          g.control = gamlss.control(trace=FALSE))
       coef.matrix2[i,] <- as.vector(coef(model2))},TRUE)
  
  try({model3 <- gamlssNP(y ~ t2+t3+t4+t5+t6+t7+t8+t9+t10+t11+t12+x1+x2, random=~1|id, family = BI(probit), 
                          data=dat, K=1, mixture="gq", tol=0.5, control = NP.control(EMcc=0.001, trace=FALSE), 
                          g.control = gamlss.control(trace=FALSE))
       coef.matrix3[i,] <- as.vector(coef(model3))},TRUE)
  
  print(i)
}

coef.matrix0<-coef.matrix0[-95,] #remove 1 NA
sink("sim5.log")

mean(coef.matrix0[,13])*sqrt(trigamma(1)+trigamma(1/5)) / (pi/sqrt(6)) #x1 cloglog
mean(coef.matrix0[,14])*sqrt(trigamma(1)+trigamma(1/5)) / (pi/sqrt(6)) #x2 cloglog
mean(coef.matrix0[,13]/coef.matrix0[,14]) #x1/x2 cloglog
mean(coef.matrix1[,13])*sqrt(trigamma(1)+trigamma(1/5)) / (pi/sqrt(3)) #x1 logit
mean(coef.matrix1[,14])*sqrt(trigamma(1)+trigamma(1/5)) / (pi/sqrt(3)) #x2 logit
mean(coef.matrix1[,13]/coef.matrix1[,14]) #x1/x2 logit
mean(coef.matrix2[,13]) #x1 pareto
mean(coef.matrix2[,14]) #x2 pareto
mean(coef.matrix2[,13]/coef.matrix2[,14]) #x1/x2 pareto
mean(coef.matrix3[,13])*sqrt(trigamma(1)+trigamma(1/5)) #x1 probit
mean(coef.matrix3[,14])*sqrt(trigamma(1)+trigamma(1/5)) #x2 probit
mean(coef.matrix3[,13]/coef.matrix3[,14]) #x1/x2 probit

sd(coef.matrix0[,13]*sqrt(trigamma(1)+trigamma(1/5)) / (pi/sqrt(6))) #x1 cloglog
sd(coef.matrix0[,14]*sqrt(trigamma(1)+trigamma(1/5)) / (pi/sqrt(6))) #x2 cloglog
sd(coef.matrix0[,13]/coef.matrix0[,14]) #x1/x2 cloglog
sd(coef.matrix1[,13]*sqrt(trigamma(1)+trigamma(1/5)) / (pi/sqrt(3))) #x1 logit
sd(coef.matrix1[,14]*sqrt(trigamma(1)+trigamma(1/5)) / (pi/sqrt(3))) #x2 logit
sd(coef.matrix1[,13]/coef.matrix1[,14]) #x1/x2 logit
sd(coef.matrix2[,13]) #x1 pareto
sd(coef.matrix2[,14]) #x2 pareto
sd(coef.matrix2[,13]/coef.matrix2[,14]) #x1/x2 pareto
sd(coef.matrix3[,13]*sqrt(trigamma(1)+trigamma(1/5))) #x1 probit
sd(coef.matrix3[,14]*sqrt(trigamma(1)+trigamma(1/5))) #x2 probit
sd(coef.matrix3[,13]/coef.matrix3[,14]) #x1/x2 probit

sink()

mean(coef.matrix0[,13]) #x1 cloglog
mean(coef.matrix0[,14]) #x2 cloglog
mean(coef.matrix0[,13]/coef.matrix0[,14]) #x1/x2 cloglog
mean(coef.matrix1[,13]) #x1 logit
mean(coef.matrix1[,14]) #x2 logit
mean(coef.matrix1[,13]/coef.matrix1[,14]) #x1/x2 logit
mean(coef.matrix2[,13]) #x1 pareto
mean(coef.matrix2[,14]) #x2 pareto
mean(coef.matrix2[,13]/coef.matrix2[,14]) #x1/x2 pareto
mean(coef.matrix3[,13]) #x1 probit
mean(coef.matrix3[,14]) #x2 probit
mean(coef.matrix3[,13]/coef.matrix3[,14]) #x1/x2 probit

sd(coef.matrix0[,13]) #x1 cloglog
sd(coef.matrix0[,14]) #x2 cloglog
sd(coef.matrix0[,13]/coef.matrix0[,14]) #x1/x2 cloglog
sd(coef.matrix1[,13]) #x1 logit
sd(coef.matrix1[,14]) #x2 logit
sd(coef.matrix1[,13]/coef.matrix1[,14]) #x1/x2 logit
sd(coef.matrix2[,13]) #x1 pareto
sd(coef.matrix2[,14]) #x2 pareto
sd(coef.matrix2[,13]/coef.matrix2[,14]) #x1/x2 pareto
sd(coef.matrix3[,13]) #x1 probit
sd(coef.matrix3[,14]) #x2 probit
sd(coef.matrix3[,13]/coef.matrix3[,14]) #x1/x2 probit

mean(coef.matrix0[,13])*sqrt(trigamma(1)+trigamma(1/5)) / (pi/sqrt(6)) #x1 cloglog
mean(coef.matrix0[,14])*sqrt(trigamma(1)+trigamma(1/5)) / (pi/sqrt(6)) #x2 cloglog
mean(coef.matrix0[,13]/coef.matrix0[,14]) #x1/x2 cloglog
mean(coef.matrix1[,13])*sqrt(trigamma(1)+trigamma(1/5)) / (pi/sqrt(3)) #x1 logit
mean(coef.matrix1[,14])*sqrt(trigamma(1)+trigamma(1/5)) / (pi/sqrt(3)) #x2 logit
mean(coef.matrix1[,13]/coef.matrix1[,14]) #x1/x2 logit
mean(coef.matrix2[,13]) #x1 pareto
mean(coef.matrix2[,14]) #x2 pareto
mean(coef.matrix2[,13]/coef.matrix2[,14]) #x1/x2 pareto
mean(coef.matrix3[,13])*sqrt(trigamma(1)+trigamma(1/5)) #x1 probit
mean(coef.matrix3[,14])*sqrt(trigamma(1)+trigamma(1/5)) #x2 probit
mean(coef.matrix3[,13]/coef.matrix3[,14]) #x1/x2 probit

sd(coef.matrix0[,13]*sqrt(trigamma(1)+trigamma(1/5)) / (pi/sqrt(6))) #x1 cloglog
sd(coef.matrix0[,14]*sqrt(trigamma(1)+trigamma(1/5)) / (pi/sqrt(6))) #x2 cloglog
sd(coef.matrix0[,13]/coef.matrix0[,14]) #x1/x2 cloglog
sd(coef.matrix1[,13]*sqrt(trigamma(1)+trigamma(1/5)) / (pi/sqrt(3))) #x1 logit
sd(coef.matrix1[,14]*sqrt(trigamma(1)+trigamma(1/5)) / (pi/sqrt(3))) #x2 logit
sd(coef.matrix1[,13]/coef.matrix1[,14]) #x1/x2 logit
sd(coef.matrix2[,13]) #x1 pareto
sd(coef.matrix2[,14]) #x2 pareto
sd(coef.matrix2[,13]/coef.matrix2[,14]) #x1/x2 pareto
sd(coef.matrix3[,13]*sqrt(trigamma(1)+trigamma(1/5))) #x1 probit
sd(coef.matrix3[,14]*sqrt(trigamma(1)+trigamma(1/5))) #x2 probit
sd(coef.matrix3[,13]/coef.matrix3[,14]) #x1/x2 probit


#Amemiya correction
xi<-c(0.000000001,1,5)
mu<-c(-1.891,-1.815,-1.492)
mu4<--1.080
c(1-(1+xi/exp(-mu))^(-1/xi),pnorm(mu4)) #should all be approx. 0.14
f<-c(exp(mu)*(1+xi*exp(mu))^(-1/xi-1),dnorm(mu4))
f
mean(coef.matrix0[,13])*f[1]/f[3] #x1 cloglog
mean(coef.matrix0[,14])*f[1]/f[3] #x2 cloglog
mean(coef.matrix0[,13]/coef.matrix0[,14]) #x1/x2 cloglog
mean(coef.matrix1[,13])*f[2]/f[3] #x1 logit
mean(coef.matrix1[,14])*f[2]/f[3] #x2 logit
mean(coef.matrix1[,13]/coef.matrix1[,14]) #x1/x2 logit
mean(coef.matrix2[,13]) #x1 pareto
mean(coef.matrix2[,14]) #x2 pareto
mean(coef.matrix2[,13]/coef.matrix2[,14]) #x1/x2 pareto
mean(coef.matrix3[,13])*f[4]/f[3] #x1 probit
mean(coef.matrix3[,14])*f[4]/f[3] #x2 probit
mean(coef.matrix3[,13]/coef.matrix3[,14]) #x1/x2 probit

sd(coef.matrix0[,13]*f[1]/f[3]) #x1 cloglog
sd(coef.matrix0[,14]*f[1]/f[3]) #x2 cloglog
sd(coef.matrix0[,13]/coef.matrix0[,14]) #x1/x2 cloglog
sd(coef.matrix1[,13]*f[2]/f[3]) #x1 logit
sd(coef.matrix1[,14]*f[2]/f[3]) #x2 logit
sd(coef.matrix1[,13]/coef.matrix1[,14]) #x1/x2 logit
sd(coef.matrix2[,13]) #x1 pareto
sd(coef.matrix2[,14]) #x2 pareto
sd(coef.matrix2[,13]/coef.matrix2[,14]) #x1/x2 pareto
sd(coef.matrix3[,13]*f[4]/f[3]) #x1 probit
sd(coef.matrix3[,14]*f[4]/f[3]) #x2 probit
sd(coef.matrix3[,13]/coef.matrix3[,14]) #x1/x2 probit


