#=========================================================================
# R-file for simulation experiments (Tab. 1)      
# author: Wolfgang Hess
# ========================================================================

library(gamlss.mx)
library(abind)

set.seed(4537269)
iterations = 100

n <- 5000 #number of individuals
k <- 12  #maximum number of periods before the spell is artificially censored
nk <- n*k
id <- rep(1:n, each=k)

for (i in 1:iterations) {
  
  x1 <- rep(rnorm(n = n, mean = 0, sd = 1), each=k)   
  x2 <- rep(rgamma(n = n, shape=1)-1, each=k)   #gamma distributed x with mean=0 and sd=1
  t <- rep(1:k, times=n)
  lnt <- log(t)  #baseline hazard
  h <- 1-(1+5/exp(-(x1+x2-lnt)))^(-1/5)   #grouped hazard
  
  y <- rep(0, times=nk)
  for (j in 1:nk) {
    if (h[j]>runif(1)) {
      y[j]<-1
    }
  }
  summary(h)
  summary(y) #means should be roughly equal!
  
  
  #generating duration dummies
  #---------------------------
  t2 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==2) {
      t2[j]<-1
    }
  }
  t3 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==3) {
      t3[j]<-1
    }
  }
  t4 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==4) {
      t4[j]<-1
    }
  }
  t5 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==5) {
      t5[j]<-1
    }
  }
  t6<- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==6) {
      t6[j]<-1
    }
  }
  t7 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==7) {
      t7[j]<-1
    }
  }
  t8 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==8) {
      t8[j]<-1
    }
  }
  t9 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==9) {
      t9[j]<-1
    }
  }
  t10 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==10) {
      t10[j]<-1
    }
  }
  t11 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==11) {
      t11[j]<-1
    }
  }
  t12 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==12) {
      t12[j]<-1
    }
  }
  
  
  #dropping obs. after the first y_it=1
  #------------------------------------
  temp <- rep(0, times=nk)
  for (l in seq(1,nk,by=k)) {
    m<-l+k-1
    temp[l:m]<-cumsum(y[l:m])
  }
  drop <- rep(0, times=nk)
  for (j in 1:nk) {
    if (temp[j]>1||(temp[j]==1&&y[j]==0)) {
      drop[j]<-1
    }
  }
  dat <- data.frame(id,y,x1,x2,t,lnt,t2,t3,t4,t5,t6,t7,t8,t9,t10,t11,t12,drop,h)
  dat <- dat[dat$drop==0,]
  assign(paste("data.set", i, sep = ""), dat)
  
  print(i)
}

rm(list=setdiff(ls(), c("iterations", ls(pattern="^data.set"))))

burr <- function(xi=1) 
{ 
  linkfun <- function(mu) log(((1-mu)^(-xi)-1)/xi) 
  linkinv <- function(eta) 1-(1+xi*exp(eta))^(-1/xi) 
  mu.eta <- function(eta) exp(eta)*(1+xi*exp(eta))^(-1/xi-1) 
  valideta <- function(eta) TRUE 
  link <- paste("burr(", xi, ")", sep="") 
  structure(list(linkfun = linkfun, linkinv = linkinv, 
                 mu.eta = mu.eta, valideta = valideta, name = link), 
            class = "link-gamlss") 
}

for (j in seq(3,7,0.5)) {
  
  assign(paste("coef.matrix", j, sep = ""), 
         matrix(data = NA, nrow = iterations, ncol = 17,
                dimnames=list(1:iterations,c("cons", "t2", "t3", "t4", "t5",
                                             "t6", "t7", "t8", "t9", "t10", 
                                             "t11", "t12", "x1", "x2", 
                                             "x1tilde", "x2tilde", "loglik"))))   }
coef.matrix.list<-list(coef.matrix3,coef.matrix3.5,coef.matrix4,coef.matrix4.5,
                       coef.matrix5,coef.matrix5.5,coef.matrix6,coef.matrix6.5,
                       coef.matrix7)

for (i in 1:iterations) {
  dat<-get(paste("data.set", i, sep = ""))
  
  for (j in seq(1,9,1)) {
    
    xi<-(j+5)/2
    try({model2 <- gamlssNP(y ~ t2+t3+t4+t5+t6+t7+t8+t9+t10+t11+t12+x1+x2, random=~1|id, family = BI(burr(xi=xi)), 
                            data=dat, K=1, mixture="gq", tol=0.5, control = NP.control(EMcc=0.001, trace=FALSE), 
                            g.control = gamlss.control(trace=FALSE))         
         
         mu<-mean(dat[,19])
         phat<-1-(1+xi/exp(-mu))^(-1/xi)
         f<-exp(mu)*(1+xi*exp(mu))^(-1/xi-1)/(exp(mu)*(1+5*exp(mu))^(-1/5-1))
         x1tilde<-f*coef(model2)[13]
         x2tilde<-f*coef(model2)[14]
         coef.matrix.list[[j]][i,]<-as.vector(c(coef(model2),x1tilde,x2tilde,-model2$G.deviance/2))},TRUE)  
    }
  
  print(i)
}

save(coef.matrix.list, file="sim9d.RData")

coef.matrix.array<-do.call(abind,c(coef.matrix.list,list(along=3)))
coef.matrix.xihat<-matrix(data = NA, nrow = iterations, ncol = 17,
                          dimnames=list(1:iterations,c("cons", "t2", "t3", "t4", "t5",
                                                       "t6", "t7", "t8", "t9", "t10", 
                                                       "t11", "t12", "x1", "x2", 
                                                       "x1tilde", "x2tilde", "loglik")))
for (i in 1:iterations){
  coef.matrix.xihat[i,]<-coef.matrix.array[i,,which.max(coef.matrix.array[i,17,])]
}

mean(coef.matrix.xihat[,13],na.rm=TRUE) #x1
mean(coef.matrix.xihat[,14],na.rm=TRUE) #x2
mean(coef.matrix.xihat[,13]/coef.matrix.xihat[,14],na.rm=TRUE) #x1/x2
mean(coef.matrix.xihat[,15],na.rm=TRUE) #x1tilde
mean(coef.matrix.xihat[,16],na.rm=TRUE) #x2tilde
mean(coef.matrix.xihat[,15]/coef.matrix.xihat[,16],na.rm=TRUE) #x1tilde/x2tilde
mean(coef.matrix.xihat[,17],na.rm=TRUE) #loglik

sd(coef.matrix.xihat[,13],na.rm=TRUE) #x1
sd(coef.matrix.xihat[,14],na.rm=TRUE) #x2
sd(coef.matrix.xihat[,13]/coef.matrix.xihat[,14],na.rm=TRUE) #x1/x2
sd(coef.matrix.xihat[,15],na.rm=TRUE) #x1tilde
sd(coef.matrix.xihat[,16],na.rm=TRUE) #x2tilde
sd(coef.matrix.xihat[,15]/coef.matrix.xihat[,16],na.rm=TRUE) #x1tilde/x2tilde
sd(coef.matrix.xihat[,17],na.rm=TRUE) #loglik



#----------------------------------------------------------------------------------

xihat<-rep(0,iterations)

for (i in 1:iterations){
  xihat[i]<-(which.max(coef.matrix.array[i,17,])+5)/2
}
xihat
mean(xihat)

coef.matrix.new<-cbind(coef.matrix.xihat,xihat)

for (i in 1:iterations) {
  xb<-cbind(1,as.matrix(get(paste("data.set", i, sep = ""))[,c(7:17,3:4)]))%*%as.vector(coef.matrix.new[i,1:14])
  phat<-1-(1+xihat[i]/exp(-xb))^(-1/xihat[i])
  htrue<-get(paste("data.set", i, sep = ""))[,19]
  
  mat<-cbind(phat, htrue)
  mat.sort<-mat[order(mat[,2]),]
  mat.sort<-cbind(mat.sort,ceiling(mat.sort[,2]*100)/100)
  assign(paste("phatmean", i, sep = ""),as.data.frame(tapply(mat.sort[,1],INDEX=mat.sort[,3],mean)))
  assign(paste("htruemean", i, sep = ""),as.data.frame(tapply(mat.sort[,2],INDEX=mat.sort[,3],mean)))
}

m0<-phatmean1
for (i in 2:iterations) {
  m0<-merge(m0,get(paste("phatmean",i,sep="")),by="row.names", all=TRUE, suffixes=c(paste(".x",i,sep=""),paste(".y",i,sep="")))
  rownames(m0)<-m0$Row.names
  m0<-m0[,-1]
}
phat<-rowMeans(m0,na.rm=TRUE)

m4<-htruemean1
for (i in 2:iterations) {
  m4<-merge(m4,get(paste("htruemean",i,sep="")),by="row.names", all=TRUE, suffixes=c(paste(".x",i,sep=""),paste(".y",i,sep="")))
  rownames(m4)<-m4$Row.names
  m4<-m4[,-1]
}
phattrue<-rowMeans(m4,na.rm=TRUE)



postscript(file="sim9d_fig1.eps", onefile=FALSE, horizontal=FALSE,
           width=6, height =5, paper="special", family="Times")
plot(phattrue, phat/phattrue, type="l",col="black",lty=2, lwd=1,
     xlab="True hazard", xaxt="l", ylab="Predicted relative hazard", ylim=c(0.8,2.5))
dev.off()

phatrel<-phat/phattrue

save(phatrel, file="phatrel.RData")

