#=========================================================================
# R-file for simulation experiments (Tab. 1)      
# author: Wolfgang Hess
# ========================================================================

library(gamlss.mx)

set.seed(4537269)
iterations = 100

n <- 1000 #number of individuals
k <- 12  #maximum number of periods before the spell is artificially censored
nk <- n*k
id <- rep(1:n, each=k)

for (i in 1:iterations) {
  
  x1 <- rep(rnorm(n = n, mean = 0, sd = 1.5), each=k)   
  het <- rep(rnorm(n = n, mean = 0, sd = 0.5), each=k)
  t <- rep(1:k, times=n)
  lnt <- log(t)  #baseline hazard
  h <- 1-(1+5/exp(-(x1+het-lnt)))^(-1/5)   #grouped hazard
  
  y <- rep(0, times=nk)
  for (j in 1:nk) {
    if (h[j]>runif(1)) {
      y[j]<-1
    }
  }
  summary(h)
  summary(y) #means should be roughly equal!
  
  
  #generating duration dummies
  #---------------------------
  t2 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==2) {
      t2[j]<-1
    }
  }
  t3 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==3) {
      t3[j]<-1
    }
  }
  t4 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==4) {
      t4[j]<-1
    }
  }
  t5 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==5) {
      t5[j]<-1
    }
  }
  t6<- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==6) {
      t6[j]<-1
    }
  }
  t7 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==7) {
      t7[j]<-1
    }
  }
  t8 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==8) {
      t8[j]<-1
    }
  }
  t9 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==9) {
      t9[j]<-1
    }
  }
  t10 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==10) {
      t10[j]<-1
    }
  }
  t11 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==11) {
      t11[j]<-1
    }
  }
  t12 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==12) {
      t12[j]<-1
    }
  }
  
  
  #dropping obs. after the first y_it=1
  #------------------------------------
  temp <- rep(0, times=nk)
  for (l in seq(1,nk,by=k)) {
    m<-l+k-1
    temp[l:m]<-cumsum(y[l:m])
  }
  drop <- rep(0, times=nk)
  for (j in 1:nk) {
    if (temp[j]>1||(temp[j]==1&&y[j]==0)) {
      drop[j]<-1
    }
  }
  dat <- data.frame(id,y,x1,t,lnt,t2,t3,t4,t5,t6,t7,t8,t9,t10,t11,t12,drop,h)
  dat <- dat[dat$drop==0,]
  assign(paste("data.set", i, sep = ""), dat)
  
  print(i)
}

#save(list=ls(pattern="^data.set"), file="sim4.RData")
rm(list=setdiff(ls(), c("iterations", ls(pattern="^data.set"))))

burr <- function(xi=1) 
{ 
  linkfun <- function(mu) log(((1-mu)^(-xi)-1)/xi) 
  linkinv <- function(eta) 1-(1+xi*exp(eta))^(-1/xi) 
  mu.eta <- function(eta) exp(eta)*(1+xi*exp(eta))^(-1/xi-1) 
  valideta <- function(eta) TRUE 
  link <- paste("burr(", xi, ")", sep="") 
  structure(list(linkfun = linkfun, linkinv = linkinv, 
                 mu.eta = mu.eta, valideta = valideta, name = link), 
            class = "link-gamlss") 
}

for (j in 0:3){
  assign(paste("coef.matrix", j, sep = ""), 
         matrix(data = NA, nrow = iterations, ncol = 14,
                dimnames=list(1:iterations,c("cons", "t2", "t3", "t4", "t5",
                           "t6", "t7", "t8", "t9", "t10", 
                           "t11", "t12", "x1", "sigma"))))
}

deriv0<-matrix(data=NA,nrow=iterations,ncol=1)
deriv1<-matrix(data=NA,nrow=iterations,ncol=1)
deriv2<-matrix(data=NA,nrow=iterations,ncol=1)
deriv3<-matrix(data=NA,nrow=iterations,ncol=1)
hr0<-matrix(data=NA,nrow=iterations,ncol=1)
hr1<-matrix(data=NA,nrow=iterations,ncol=1)
hr2<-matrix(data=NA,nrow=iterations,ncol=1)
hr3<-matrix(data=NA,nrow=iterations,ncol=1)
deriv0b<-matrix(data=NA,nrow=iterations,ncol=1)
deriv1b<-matrix(data=NA,nrow=iterations,ncol=1)
deriv2b<-matrix(data=NA,nrow=iterations,ncol=1)
deriv3b<-matrix(data=NA,nrow=iterations,ncol=1)
hr0b<-matrix(data=NA,nrow=iterations,ncol=1)
hr1b<-matrix(data=NA,nrow=iterations,ncol=1)
hr2b<-matrix(data=NA,nrow=iterations,ncol=1)
hr3b<-matrix(data=NA,nrow=iterations,ncol=1)


for (i in 1:iterations) {
  dat<-get(paste("data.set", i, sep = ""))
  
  try({model0 <- gamlssNP(y ~ t2+t3+t4+t5+t6+t7+t8+t9+t10+t11+t12+x1, random=~1|id, family = BI(cloglog), 
                      data=dat, K=9, mixture="gq", tol=0.5, control = NP.control(EMcc=0.001, trace=FALSE), 
                      g.control = gamlss.control(trace=FALSE))
  coef.matrix0[i,] <- as.vector(coef(model0))       
       xb0<-mean(cbind(1,1,as.matrix(dat[,3]))%*%coef(model0)[c(1:2,13)])
       phat0<-1-exp(-exp(xb0))
       deriv0[i] <- coef(model0)["x1"]*exp(xb0)*exp(-exp(xb0))/phat0
       hr0[i] <- (1-exp(-exp(xb0+coef(model0)["x1"])))/phat0  
       xb0<-mean(cbind(1,1,as.matrix(dat[,3]))%*%coef(model0)[c(1,12:13)])
       phat0<-1-exp(-exp(xb0))
       deriv0b[i] <- coef(model0)["x1"]*exp(xb0)*exp(-exp(xb0))/phat0
       hr0b[i] <- (1-exp(-exp(xb0+coef(model0)["x1"])))/phat0},TRUE)
  
  try({model1 <- gamlssNP(y ~ t2+t3+t4+t5+t6+t7+t8+t9+t10+t11+t12+x1, random=~1|id, family = BI(logit), 
                          data=dat, K=9, mixture="gq", tol=0.5, control = NP.control(EMcc=0.001, trace=FALSE), 
                          g.control = gamlss.control(trace=FALSE))
       coef.matrix1[i,] <- as.vector(coef(model1))
       xb1<-mean(cbind(1,1,as.matrix(dat[,3]))%*%coef(model1)[c(1:2,13)])
       phat1<-1/(1+exp(-xb1))
       deriv1[i] <- coef(model1)["x1"]*exp(xb1)*(1+exp(xb1))^(-2)/phat1
       hr1[i] <- (1/(1+exp(-(xb1+coef(model1)["x1"]))))/phat1
       xb1<-mean(cbind(1,1,as.matrix(dat[,3]))%*%coef(model1)[c(1,12:13)])
       phat1<-1/(1+exp(-xb1))
       deriv1b[i] <- coef(model1)["x1"]*exp(xb1)*(1+exp(xb1))^(-2)/phat1
       hr1b[i] <- (1/(1+exp(-(xb1+coef(model1)["x1"]))))/phat1},TRUE)
  
  try({model2 <- gamlssNP(y ~ t2+t3+t4+t5+t6+t7+t8+t9+t10+t11+t12+x1, random=~1|id, family = BI(burr(xi=5)), 
                          data=dat, K=9, mixture="gq", tol=0.5, control = NP.control(EMcc=0.001, trace=FALSE), 
                          g.control = gamlss.control(trace=FALSE))
       coef.matrix2[i,] <- as.vector(coef(model2))       
       xb2<-mean(cbind(1,1,as.matrix(dat[,3]))%*%coef(model2)[c(1:2,13)])
       phat2<-1-(1+5/exp(-xb2))^(-1/5)
       deriv2[i] <- coef(model2)["x1"]*exp(xb2)*(1+5*exp(xb2))^(-1/5-1)/phat2
       hr2[i] <- (1-(1+5*exp(xb2+coef(model2)["x1"]))^(-1/5))/phat2
       xb2<-mean(cbind(1,1,as.matrix(dat[,3]))%*%coef(model2)[c(1,12:13)])
       phat2<-1-(1+5/exp(-xb2))^(-1/5)
       deriv2b[i] <- coef(model2)["x1"]*exp(xb2)*(1+5*exp(xb2))^(-1/5-1)/phat2
       hr2b[i] <- (1-(1+5*exp(xb2+coef(model2)["x1"]))^(-1/5))/phat2},TRUE)
  
  try({model3 <- gamlssNP(y ~ t2+t3+t4+t5+t6+t7+t8+t9+t10+t11+t12+x1, random=~1|id, family = BI(probit), 
                          data=dat, K=9, mixture="gq", tol=0.5, control = NP.control(EMcc=0.001, trace=FALSE), 
                          g.control = gamlss.control(trace=FALSE))
       coef.matrix3[i,] <- as.vector(coef(model3))       
       xb3<-mean(cbind(1,1,as.matrix(dat[,3]))%*%coef(model3)[c(1:2,13)])
       phat3<-pnorm(xb3)
       deriv3[i] <- coef(model3)["x1"]*dnorm(xb3)/phat3
       hr3[i] <- pnorm(xb3+coef(model3)["x1"])/phat3       
       xb3<-mean(cbind(1,1,as.matrix(dat[,3]))%*%coef(model3)[c(1,12:13)])
       phat3<-pnorm(xb3)
       deriv3b[i] <- coef(model3)["x1"]*dnorm(xb3)/phat3
       hr3b[i] <- pnorm(xb3+coef(model3)["x1"])/phat3},TRUE)
  
  print(i)
}

coef.matrix0[,1]
coef.matrix1[,1]
coef.matrix2[,1]
coef.matrix3[,1]

mean(coef.matrix0[,13],na.rm=TRUE) #x1 cloglog
mean(coef.matrix0[,14],na.rm=TRUE) #het cloglog
mean(coef.matrix0[,13]/coef.matrix0[,14],na.rm=TRUE) #x1/het cloglog
mean(coef.matrix1[,13],na.rm=TRUE) #x1 logit
mean(coef.matrix1[,14],na.rm=TRUE) #het logit
mean(coef.matrix1[,13]/coef.matrix1[,14],na.rm=TRUE) #x1/het logit
mean(coef.matrix2[,13],na.rm=TRUE) #x1 pareto
mean(coef.matrix2[,14],na.rm=TRUE) #het pareto
mean(coef.matrix2[,13]/coef.matrix2[,14],na.rm=TRUE) #x1/het pareto
mean(coef.matrix3[,13],na.rm=TRUE) #x1 probit
mean(coef.matrix3[,14],na.rm=TRUE) #het probit
mean(coef.matrix3[,13]/coef.matrix3[,14],na.rm=TRUE) #x1/het probit

sd(coef.matrix0[,13],na.rm=TRUE) #x1 cloglog
sd(coef.matrix0[,14],na.rm=TRUE) #het cloglog
sd(coef.matrix0[,13]/coef.matrix0[,14],na.rm=TRUE) #x1/het cloglog
sd(coef.matrix1[,13],na.rm=TRUE) #x1 logit
sd(coef.matrix1[,14],na.rm=TRUE) #het logit
sd(coef.matrix1[,13]/coef.matrix1[,14],na.rm=TRUE) #x1/het logit
sd(coef.matrix2[,13],na.rm=TRUE) #x1 pareto
sd(coef.matrix2[,14],na.rm=TRUE) #het pareto
sd(coef.matrix2[,13]/coef.matrix2[,14],na.rm=TRUE) #x1/het pareto
sd(coef.matrix3[,13],na.rm=TRUE) #x1 probit
sd(coef.matrix3[,14],na.rm=TRUE) #het probit
sd(coef.matrix3[,13]/coef.matrix3[,14],na.rm=TRUE) #x1/het probit

mean(deriv0[,1],na.rm=TRUE)
mean(deriv1[,1],na.rm=TRUE)
mean(deriv2[,1],na.rm=TRUE)
mean(deriv3[,1],na.rm=TRUE)
sd(deriv0[,1],na.rm=TRUE)
sd(deriv1[,1],na.rm=TRUE)
sd(deriv2[,1],na.rm=TRUE)
sd(deriv3[,1],na.rm=TRUE)

mean(deriv0b[,1],na.rm=TRUE)
mean(deriv1b[,1],na.rm=TRUE)
mean(deriv2b[,1],na.rm=TRUE)
mean(deriv3b[,1],na.rm=TRUE)
sd(deriv0b[,1],na.rm=TRUE)
sd(deriv1b[,1],na.rm=TRUE)
sd(deriv2b[,1],na.rm=TRUE)
sd(deriv3b[,1],na.rm=TRUE)

mean(hr0[,1],na.rm=TRUE)
mean(hr1[,1],na.rm=TRUE)
mean(hr2[,1],na.rm=TRUE)
mean(hr3[,1],na.rm=TRUE)
sd(hr0[,1],na.rm=TRUE)
sd(hr1[,1],na.rm=TRUE)
sd(hr2[,1],na.rm=TRUE)
sd(hr3[,1],na.rm=TRUE)

mean(hr0b[,1],na.rm=TRUE)
mean(hr1b[,1],na.rm=TRUE)
mean(hr2b[,1],na.rm=TRUE)
mean(hr3b[,1],na.rm=TRUE)
sd(hr0b[,1],na.rm=TRUE)
sd(hr1b[,1],na.rm=TRUE)
sd(hr2b[,1],na.rm=TRUE)
sd(hr3b[,1],na.rm=TRUE)

