#=========================================================================
# R-file for simulation experiments (Tab. 1)      
# author: Wolfgang Hess
# ========================================================================


library(gamlss.mx)
library(abind)

set.seed(4537269)
iterations = 100

n <- 5000 #number of individuals
k <- 12  #maximum number of periods before the spell is artificially censored
nk <- n*k
id <- rep(1:n, each=k)

for (i in 1:iterations) {
  
  x1 <- rep(rnorm(n = n, mean = 0, sd = 1.5), each=k)   
  het <- rep(rnorm(n = n, mean = 0, sd = 0.5), each=k)
  t <- rep(1:k, times=n)
  lnt <- log(t)  #baseline hazard
  h <- 1-(1+5/exp(-(x1+het-lnt)))^(-1/5)   #grouped hazard
  
  y <- rep(0, times=nk)
  for (j in 1:nk) {
    if (h[j]>runif(1)) {
      y[j]<-1
    }
  }
  summary(h)
  summary(y) #means should be roughly equal!
  
  
  #generating duration dummies
  #---------------------------
  t2 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==2) {
      t2[j]<-1
    }
  }
  t3 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==3) {
      t3[j]<-1
    }
  }
  t4 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==4) {
      t4[j]<-1
    }
  }
  t5 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==5) {
      t5[j]<-1
    }
  }
  t6<- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==6) {
      t6[j]<-1
    }
  }
  t7 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==7) {
      t7[j]<-1
    }
  }
  t8 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==8) {
      t8[j]<-1
    }
  }
  t9 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==9) {
      t9[j]<-1
    }
  }
  t10 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==10) {
      t10[j]<-1
    }
  }
  t11 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==11) {
      t11[j]<-1
    }
  }
  t12 <- rep(0, times=nk)
  for (j in 1:nk) {
    if (t[j]==12) {
      t12[j]<-1
    }
  }
  
  
  #dropping obs. after the first y_it=1
  #------------------------------------
  temp <- rep(0, times=nk)
  for (l in seq(1,nk,by=k)) {
    m<-l+k-1
    temp[l:m]<-cumsum(y[l:m])
  }
  drop <- rep(0, times=nk)
  for (j in 1:nk) {
    if (temp[j]>1||(temp[j]==1&&y[j]==0)) {
      drop[j]<-1
    }
  }
  dat <- data.frame(id,y,x1,t,lnt,t2,t3,t4,t5,t6,t7,t8,t9,t10,t11,t12,drop,h)
  dat <- dat[dat$drop==0,]
  assign(paste("data.set", i, sep = ""), dat)
  
  print(i)
}

rm(list=setdiff(ls(), c("iterations", ls(pattern="^data.set"))))

burr <- function(xi=1) 
{ 
  linkfun <- function(mu) log(((1-mu)^(-xi)-1)/xi) 
  linkinv <- function(eta) 1-(1+xi*exp(eta))^(-1/xi) 
  mu.eta <- function(eta) exp(eta)*(1+xi*exp(eta))^(-1/xi-1) 
  valideta <- function(eta) TRUE 
  link <- paste("burr(", xi, ")", sep="") 
  structure(list(linkfun = linkfun, linkinv = linkinv, 
                 mu.eta = mu.eta, valideta = valideta, name = link), 
            class = "link-gamlss") 
}

for (j in seq(3,7,0.5)) {
  
  assign(paste("coef.matrix", j, sep = ""), 
         matrix(data = NA, nrow = iterations, ncol = 17,
                dimnames=list(1:iterations,c("cons", "t2", "t3", "t4", "t5",
                                             "t6", "t7", "t8", "t9", "t10", 
                                             "t11", "t12", "x1", "sigma", 
                                             "hr2", "hr2b", "loglik"))))   }
coef.matrix.list<-list(coef.matrix3,coef.matrix3.5,coef.matrix4,coef.matrix4.5,
                       coef.matrix5,coef.matrix5.5,coef.matrix6,coef.matrix6.5,
                       coef.matrix7)

for (i in 1:iterations) {
  dat<-get(paste("data.set", i, sep = ""))
  
  for (j in seq(1,9,1)) {
    
    xi<-(j+5)/2
    try({model2 <- gamlssNP(y ~ t2+t3+t4+t5+t6+t7+t8+t9+t10+t11+t12+x1, random=~1|id, family = BI(burr(xi=xi)), 
                            data=dat, K=9, mixture="gq", tol=0.5, control = NP.control(EMcc=0.001, trace=FALSE), 
                            g.control = gamlss.control(trace=FALSE))         
         
         xb2<-mean(cbind(1,1,as.matrix(dat[,3]))%*%coef(model2)[c(1:2,13)])
         phat2<-1-(1+xi/exp(-xb2))^(-1/xi)
         hr2 <- (1-(1+xi*exp(xb2+coef(model2)["x1"]))^(-1/xi))/phat2
         xb2<-mean(cbind(1,1,as.matrix(dat[,3]))%*%coef(model2)[c(1,12:13)])
         phat2<-1-(1+xi/exp(-xb2))^(-1/xi)
         hr2b <- (1-(1+xi*exp(xb2+coef(model2)["x1"]))^(-1/xi))/phat2
         coef.matrix.list[[j]][i,]<-as.vector(c(coef(model2),hr2,hr2b,-model2$G.deviance/2))},TRUE)  
    }
  
  print(i)
}

save(coef.matrix.list, file="sim8d.RData")

coef.matrix.array<-do.call(abind,c(coef.matrix.list,list(along=3)))
coef.matrix.xihat<-matrix(data = NA, nrow = iterations, ncol = 17,
                          dimnames=list(1:iterations,c("cons", "t2", "t3", "t4", "t5",
                                                       "t6", "t7", "t8", "t9", "t10", 
                                                       "t11", "t12", "x1", "sigma", 
                                                       "hr2", "hr2b", "loglik")))
for (i in 1:iterations){
  coef.matrix.xihat[i,]<-coef.matrix.array[i,,which.max(coef.matrix.array[i,17,])]
}

mean(coef.matrix.xihat[,13],na.rm=TRUE) #x
mean(coef.matrix.xihat[,14],na.rm=TRUE) #het
mean(coef.matrix.xihat[,15],na.rm=TRUE) #hr at t=1
mean(coef.matrix.xihat[,16],na.rm=TRUE) #hr at t=12
mean(coef.matrix.xihat[,17],na.rm=TRUE) #loglik

sd(coef.matrix.xihat[,13],na.rm=TRUE) #x
sd(coef.matrix.xihat[,14],na.rm=TRUE) #het
sd(coef.matrix.xihat[,15],na.rm=TRUE) #hr at t=1
sd(coef.matrix.xihat[,16],na.rm=TRUE) #hr at t=12
sd(coef.matrix.xihat[,17],na.rm=TRUE) #loglik
