/* ======================================================================= */
/* do-file evaluating the duration of bilateral trade relationships        */
/* Stata version 10.1                                                      */
/* author: Wolfgang Hess                                                   */
/* ======================================================================= */

version 10.1
clear
set more off
set mem 600m
use pareto_data.dta





**********************
***** Estimation *****
**********************

* conventional cloglog
*---------------------
cloglog y t2-t14 t15_16 transport gdp tariff pctrer_relative cov_uv spelldum agriculture ///
reference_priced homogeneous

* conventional probit 
*--------------------
probit y t2-t14 t15_16 transport gdp tariff pctrer_relative cov_uv spelldum agriculture ///
reference_priced homogeneous

* conventional logit (also used to get starting values for Pareto model below
*----------------------------------------------------------------------------
logit y t2-t14 t15_16 transport gdp tariff pctrer_relative cov_uv spelldum agriculture ///
reference_priced homogeneous
mat Init=e(b)*1

* Pareto without heterogeneity
*-----------------------------
capture program drop paretohaz_d1
program define paretohaz_d1

	args todo b lnf g
	
		tempname xi
		tempvar xb lnxi h h1 h2

		mleval `xb' = `b'
		mleval `lnxi' = `b', scalar eq(2)		
				
		quietly {		
			scalar `xi'=exp(`lnxi')
			gen double `h'=1-(1+`xi'*exp(`xb'))^(-1/`xi') /*Pareto link*/
			gen double `h1'=exp(`xb')*(1+`xi'*exp(`xb'))^(-1/`xi'-1) /*first derivative of h w.r.t. xb*/	
			gen double `h2'=-`xi'*(1+`xi'*exp(`xb'))^(-1/`xi')*(1/`xi'^2*ln(1+`xi'*exp(`xb')) ///
							-1/`xi'*exp(`xb')/(1+`xi'*exp(`xb'))) /*first derivative of h w.r.t. ln(xi)*/
			}
		
			mlsum `lnf' = $ML_y1*ln(`h')+(1-$ML_y1)*ln(1-`h')
			if (`todo'==0|`lnf'>=.) exit
		
		tempname g1 g2
			mlvecsum `lnf' `g1' = $ML_y1*`h1'/`h'-(1-$ML_y1)*`h1'/(1-`h'), eq(1)
			mlvecsum `lnf' `g2' = $ML_y1*`h2'/`h'-(1-$ML_y1)*`h2'/(1-`h'), eq(2)
			matrix `g' = (`g1',`g2')
		
end

ml model d1 paretohaz_d1 (y = t2-t14 t15_16 transport gdp tariff pctrer_relative cov_uv spelldum agriculture ///
reference_priced homogeneous) /lnxi, technique(dfp)
ml check
matrix start = Init, 0
ml init start, copy
ml maximize, difficult trace search(off) /*nonrtol ltol(1e-7) tol(1e-5)*/
mat b=e(b)







